"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chokidar = __importStar(require("chokidar"));
const ora_1 = __importDefault(require("ora"));
const Workspace_1 = __importDefault(require("../core/Workspace"));
const RestClient_1 = __importDefault(require("../core/HTTP/RestClient"));
const wipe_1 = __importDefault(require("./wipe"));
const helpers_1 = require("../helpers");
async function asyncForEach(array, callback) {
    for (let index = 0; index < array.length; index++) {
        await callback(array[index], index, array);
    }
}
function buildFilesObject(fileObj, files, filterBy) {
    if (filterBy === 'configs') {
        fileObj.configs = files.filter((file) => {
            return !(/\//.test(file.resource.path || ''));
        });
        files = files.filter((file) => {
            return /\//.test(file.resource.path || '');
        });
        return files;
    }
    fileObj[filterBy] = files.filter((file) => {
        var _a;
        return (_a = file.resource.path) === null || _a === void 0 ? void 0 : _a.startsWith(filterBy);
    });
    files = files.filter((file) => {
        var _a;
        return !((_a = file.resource.path) === null || _a === void 0 ? void 0 : _a.startsWith(filterBy));
    });
    return files;
}
async function Deploy(workspace, path) {
    console.log(`Deploying ${workspace.name}:`);
    console.log(``);
    const spinner = (0, ora_1.default)({
        prefixText: `Deploying ${path || 'file'}...`,
        text: `reading files...`,
    }).start();
    const client = new RestClient_1.default(workspace.config, workspace.name);
    try {
        await workspace.scan();
        const fileObj = {};
        let files = workspace.files;
        files = buildFilesObject(fileObj, files, 'configs');
        files = buildFilesObject(fileObj, files, 'content');
        files = buildFilesObject(fileObj, files, 'specs');
        files = buildFilesObject(fileObj, files, 'emails');
        files = buildFilesObject(fileObj, files, 'themes');
        if (files.length > 0) {
            fileObj['other files'] = files;
        }
        await asyncForEach(Object.keys(fileObj), async (fileType) => {
            if (fileType === 'specs' && workspace.config.ignoreSpecs) {
                return;
            }
            let files = fileObj[fileType];
            if (workspace.config.skipPaths && workspace.config.skipPaths.length > 0) {
                files = fileObj[fileType].filter((file) => !workspace.config.skipPaths.some((path) => { var _a; return (_a = file.resource.path) === null || _a === void 0 ? void 0 : _a.startsWith(path); }));
            }
            spinner.prefixText = `Deploying ${fileType}`;
            for (const file of files) {
                if (path && file.location.split(path)[1] !== '') {
                    continue;
                }
                spinner.text = file.resource.path || file.resource.id || 'file';
                await file.read();
                await client.saveFile(file.resource);
            }
            spinner.prefixText = 'Deployed';
            spinner.succeed(fileType).start();
        });
        spinner.prefixText = '';
        if (!path) {
            spinner.text = workspace.config.ignoreSpecs
                ? `Deployed all non-spec files to ${workspace.name}`
                : `Deployed all files to ${workspace.name}`;
        }
        spinner.succeed();
        console.log(``);
    }
    catch (e) {
        spinner.text = client.handleError(e);
        spinner.fail();
        throw e;
    }
}
exports.default = async (args) => {
    if (!args.preserve) {
        await (0, wipe_1.default)(args);
    }
    let workspace;
    try {
        workspace = await Workspace_1.default.init(args.workspace, args.disableSSLVerification, args.ignoreSpecs, args.skipPath);
    }
    catch (e) {
        return (0, helpers_1.MissingWorkspaceError)(args.workspace);
    }
    await Deploy(workspace);
    if (args.watch) {
        console.log(`Watching`, `${workspace.path}/*`);
        console.log(``);
        const watcher = chokidar.watch('.', {
            cwd: workspace.path,
            alwaysStat: true,
        });
        watcher.on('change', (path, stats) => {
            if (stats && stats.isFile()) {
                Deploy(workspace, path);
            }
        });
    }
};
