"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const upath_1 = require("upath");
const isbinaryfile_1 = require("isbinaryfile");
const File_1 = __importDefault(require("../core/File"));
const Workspace_1 = __importDefault(require("../core/Workspace"));
const RestClient_1 = __importDefault(require("../core/HTTP/RestClient"));
const helpers_1 = require("../helpers");
async function shouldRewriteFile(resource, file, keepEncode) {
    var _a;
    const shasum = await file.getShaSum();
    if (shasum !== resource.checksum) {
        return true;
    }
    if (keepEncode && (await (0, isbinaryfile_1.isBinaryFile)(file.location))) {
        return true;
    }
    if (!keepEncode && file.isBase64Asset()) {
        await file.read();
        if ((_a = file.resource.contents) === null || _a === void 0 ? void 0 : _a.startsWith('data:')) {
            return true;
        }
    }
    return false;
}
exports.default = async (args) => {
    let workspace;
    try {
        workspace = await Workspace_1.default.init(args.workspace, args.disableSSLVerification, args.ignoreSpecs);
    }
    catch (e) {
        return (0, helpers_1.MissingWorkspaceError)(args.workspace);
    }
    const client = new RestClient_1.default(workspace.config, workspace.name);
    let response;
    try {
        response = await client.getAllFiles();
    }
    catch (e) {
        console.log(e);
        return;
    }
    let added = 0;
    let modified = 0;
    let specs = 0;
    if (response) {
        for (const resource of response) {
            if (workspace.config.ignoreSpecs && resource.path.startsWith('specs')) {
                specs++;
                continue;
            }
            const path = (0, upath_1.join)(workspace.path, resource.path);
            const file = new File_1.default(path, workspace.path);
            if (await file.exists()) {
                if (await shouldRewriteFile(resource, file, args.keepEncode)) {
                    file.write(resource.contents);
                    console.log('\t', 'Modified:', resource.path);
                    modified++;
                }
            }
            else {
                file.write(resource.contents);
                console.log('\t', 'Added:', resource.path);
                added++;
            }
        }
    }
    console.log('\t', `Fetched ${response.length} files`);
    if (specs) {
        console.log('\t', `Ignored ${specs} specs`);
    }
    if (!modified || added) {
        console.log('\t', 'No changes.');
    }
    console.log('');
    console.log('Done.');
};
