"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chokidar = __importStar(require("chokidar"));
const ora_1 = __importDefault(require("ora"));
const Workspace_1 = __importDefault(require("../core/Workspace"));
const RestClient_1 = __importDefault(require("../core/HTTP/RestClient"));
const helpers_1 = require("../helpers");
function buildFilesObject(fileGroups, files, filterBy) {
    if (filterBy === "configs") {
        const { matched, mismatched } = (0, helpers_1.arrayPartition)(files, (file) => !/\//.test(file.resource.path || ''));
        fileGroups.configs = matched;
        files = mismatched;
        return files;
    }
    const { matched, mismatched } = (0, helpers_1.arrayPartition)(files, (file) => { var _a; return (_a = file.resource.path) === null || _a === void 0 ? void 0 : _a.startsWith(filterBy); });
    fileGroups[filterBy] = matched;
    return mismatched;
}
async function Sync(workspace, args, options = {}) {
    var _a;
    console.log(`Syncing ${workspace.name}:\n`);
    const verbose = (...verboseArgs) => args.verbose && console.debug(...verboseArgs);
    const spinner = (0, ora_1.default)({
        prefixText: `Syncing ${options.path || 'all workspace files'}...`,
        text: `reading files...`,
    }).start();
    const client = new RestClient_1.default(workspace.config, workspace.name);
    try {
        const upstreamFiles = await client.getAllFiles({ fields: 'path,checksum' });
        const allFiles = await workspace.scan();
        spinner.text = `Found ${allFiles.length} files locally and ${upstreamFiles.length} files upstream`;
        spinner.succeed().start();
        const fileGroups = {};
        let remainingFiles = [...allFiles];
        remainingFiles = buildFilesObject(fileGroups, remainingFiles, "configs");
        remainingFiles = buildFilesObject(fileGroups, remainingFiles, "content");
        remainingFiles = buildFilesObject(fileGroups, remainingFiles, "specs");
        remainingFiles = buildFilesObject(fileGroups, remainingFiles, "emails");
        remainingFiles = buildFilesObject(fileGroups, remainingFiles, "themes");
        if (allFiles.length > 0) {
            fileGroups["others"] = remainingFiles;
        }
        const updatedPaths = [];
        const deletedPaths = [];
        for (const [group, files] of Object.entries(fileGroups)) {
            if (group === "specs" && workspace.config.ignoreSpecs) {
                return;
            }
            let filesToCompare = files;
            if ((_a = workspace.config.skipPaths) === null || _a === void 0 ? void 0 : _a.length) {
                filesToCompare = filesToCompare.filter((file) => !workspace.config.skipPaths.some((path) => { var _a; return (_a = file.resource.path) === null || _a === void 0 ? void 0 : _a.startsWith(path); }));
            }
            for (const file of filesToCompare) {
                const upstreamFile = upstreamFiles.find((upstreamFile) => file.resource.path === upstreamFile.path);
                const localChecksum = await file.getShaSum();
                const upstreamChecksum = upstreamFile === null || upstreamFile === void 0 ? void 0 : upstreamFile.checksum;
                if (upstreamChecksum && localChecksum === upstreamChecksum) {
                    verbose(`[${upstreamFile.path}] file up to date`);
                    continue;
                }
                if (!file.resource.path) {
                    continue;
                }
                spinner.text = file.resource.path || 'file';
                await client.saveFile(file.resource);
                updatedPaths.push(file.resource.path);
                verbose(`[${file.resource.path}] updated`);
            }
            const upstreamFilesToDelete = upstreamFiles.filter((file) => {
                var _a;
                if (!file.path) {
                    return false;
                }
                if (!file.path.startsWith(`${group}/`)) {
                    return false;
                }
                if ((_a = workspace.config.skipPaths) === null || _a === void 0 ? void 0 : _a.some((path) => { var _a; return (_a = file.path) === null || _a === void 0 ? void 0 : _a.startsWith(path); })) {
                    return false;
                }
                if (files.find((f) => f.resource.path === file.path)) {
                    return false;
                }
                return !updatedPaths.includes(file.path);
            });
            for (const file of upstreamFilesToDelete) {
                if (!file.path) {
                    continue;
                }
                spinner.text = file.path;
                await client.deleteFile(file);
                deletedPaths.push(file.path);
            }
            spinner.prefixText = 'Synchronized';
            spinner.succeed(group).start();
        }
        spinner.prefixText = '';
        spinner.text = [
            `Synchronized all ${allFiles.length} (${updatedPaths.length} updated, ${deletedPaths.length} deleted)`,
            workspace.config.ignoreSpecs && 'non-spec',
            'changes',
            options.path && `inside ${options.path}`,
            `to ${workspace.name}`,
        ]
            .filter(Boolean)
            .join(' ');
        spinner.succeed();
        console.log('');
    }
    catch (e) {
        spinner.text = client.handleError(e);
        spinner.fail();
        throw e;
    }
}
exports.default = async (args) => {
    let workspace;
    try {
        workspace = await Workspace_1.default.init(args.workspace, args.disableSSLVerification, args.ignoreSpecs, args.skipPath);
    }
    catch (e) {
        return (0, helpers_1.MissingWorkspaceError)(args.workspace);
    }
    await Sync(workspace, args);
    if (args.watch) {
        console.log(`Watching`, `${workspace.path}/*`);
        console.log(``);
        const watcher = chokidar.watch('.', {
            cwd: workspace.path,
            alwaysStat: true,
        });
        watcher.on('change', (path, stats) => {
            if (stats && stats.isFile()) {
                Sync(workspace, args, { path });
            }
        });
    }
};
