"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs-extra"));
const crypto = __importStar(require("crypto-promise"));
const upath_1 = require("upath");
const FileResource_1 = __importDefault(require("./HTTP/Resources/FileResource"));
const isbinaryfile_1 = require("isbinaryfile");
class File {
    constructor(location, workspacePath, options = {
        encoding: 'utf8',
    }) {
        this.location = (0, upath_1.toUnix)(location);
        this.encoding = options.encoding || 'utf8';
        this.workspacePath = (0, upath_1.toUnix)(workspacePath);
        this.resource = new FileResource_1.default({
            path: this.location.replace(`${this.workspacePath}/`, ''),
            contents: '',
        });
    }
    async write(contents) {
        if (this.isBase64Path() && contents.startsWith('data:')) {
            return await this.write64(contents);
        }
        return await fs.outputFile(this.location, contents, this.encoding);
    }
    async read() {
        if (this.isBase64Asset()) {
            this.resource.contents = await this.read64();
            return;
        }
        this.resource.contents = await fs.readFile(this.location, this.encoding);
    }
    async read64() {
        const fileExtMatch = this.location.match(/\w+$/);
        let fileExt = 'unknown';
        if (fileExtMatch) {
            fileExt = fileExtMatch[0];
        }
        const encoded = await fs.readFile(this.location, { encoding: 'base64' });
        return `data:image/${fileExt};base64,${encoded}`;
    }
    async write64(contents) {
        contents = contents.split(';base64,')[1];
        return await fs.outputFile(this.location, Buffer.from(contents, 'base64'), this.encoding);
    }
    async exists() {
        return await fs.exists(this.location);
    }
    async getShaSum(algorithm = '256') {
        await this.read();
        const buffer = await crypto.hash('sha' + algorithm)(this.resource.contents);
        return buffer.toString('hex');
    }
    isBase64Path() {
        return /themes\/\w*\/assets\//.test(this.location);
    }
    isBase64Asset() {
        return /themes\/\w*\/assets\//.test(this.location) && (0, isbinaryfile_1.isBinaryFileSync)(this.location);
    }
}
exports.default = File;
