"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const recursive_readdir_async_1 = require("recursive-readdir-async");
const fs = __importStar(require("fs-extra"));
const fs_1 = require("fs");
const upath_1 = require("upath");
const WorkspaceConfig_1 = __importDefault(require("./WorkspaceConfig"));
const WorkspaceTheme_1 = __importDefault(require("./WorkspaceTheme"));
const Config_1 = __importDefault(require("./Config"));
const File_1 = __importDefault(require("./File"));
class Workspace {
    constructor(name) {
        this.files = [];
        this.name = name;
        this.path = Workspace.getDirectoryPath(name);
        this.config = new WorkspaceConfig_1.default(this.path, 'cli.conf.yaml');
        this.portalConfig = new Config_1.default(this.path, 'portal.conf.yaml');
        this.routerConfig = new Config_1.default(this.path, 'router.conf.yaml');
    }
    static async init(name, disableSSLVerification, ignoreSpecs, skipPath) {
        if ((await this.exists(name)) === false) {
            throw new Error();
        }
        const workspace = new Workspace(name);
        await workspace.config.load();
        await workspace.portalConfig.load();
        await workspace.routerConfig.load();
        if (process.env.KONG_ADMIN_URL) {
            workspace.config.data.kong_admin_url = process.env.KONG_ADMIN_URL;
        }
        if (process.env.KONG_ADMIN_TOKEN_FILE) {
            if ((await this.exists(process.env.KONG_ADMIN_TOKEN_FILE)) === false) {
                console.log(`File specified by KONG_ADMIN_TOKEN_FILE is missing at ${process.env.KONG_ADMIN_TOKEN_FILE}`);
                console.log(``);
                throw new Error();
            }
            workspace.config.data.kong_admin_token = (0, fs_1.readFileSync)(process.env.KONG_ADMIN_TOKEN_FILE, 'utf-8').trim();
        }
        else {
            if (process.env.KONG_ADMIN_TOKEN) {
                workspace.config.data.kong_admin_token = process.env.KONG_ADMIN_TOKEN;
            }
        }
        if (disableSSLVerification) {
            workspace.config.data.disable_ssl_verification = true;
        }
        if (ignoreSpecs) {
            workspace.config.data.ignore_specs = true;
        }
        if (skipPath && skipPath.length > 0) {
            workspace.config.data.skip_paths = skipPath;
        }
        return workspace;
    }
    async scan() {
        const files = await (0, recursive_readdir_async_1.list)(this.path, { exclude: ['.DS_Store', 'cli.conf.yaml'] });
        if (!files || !Array.isArray(files)) {
            throw new Error(`Unable to scan directories: ${files.error || 'unknown error'}`);
        }
        this.files = files.map((file) => new File_1.default(file.fullname, this.path));
        return this.files;
    }
    static async exists(name) {
        try {
            const stat = await fs.lstat(Workspace.getDirectoryPath(name));
            return stat && stat.isDirectory();
        }
        catch (e) {
            return false;
        }
    }
    static getDirectoryPath(name) {
        return (0, upath_1.join)(process.cwd(), 'workspaces', name);
    }
    async getThemes() {
        const workspaceThemes = [];
        let themes = await (0, recursive_readdir_async_1.list)((0, upath_1.join)(this.path, 'themes'), {
            recursive: false,
            ignoreFolders: false,
        });
        themes = themes.filter((element) => element.isDirectory);
        for (const theme of themes) {
            workspaceThemes.push(await this.getTheme(theme.name));
        }
        return workspaceThemes;
    }
    getCurrentThemeName() {
        return this.portalConfig.data.theme.name;
    }
    async getTheme(name) {
        return await WorkspaceTheme_1.default.init(this.path, name);
    }
    async getCurrentTheme() {
        return this.getTheme(this.getCurrentThemeName());
    }
}
exports.default = Workspace;
